package shadow

import (
	"fmt"
	"github.com/xtaci/smux"
	"io"
	"net"
	"testing"
)

func Test(t *testing.T) {
	listen, err := net.Listen("tcp", "0.0.0.0:11222")
	if err != nil {
		fmt.Printf("Start server failed : %v\n", err)
		return
	}
	defer listen.Close()
	for {
		conn, err := listen.Accept()
		if err != nil {
			fmt.Printf("Accept error: %v\n", err)
			continue
		}
		fmt.Println("A")
		go handler(conn)
	}
}

func handler(conn net.Conn) {

	fakeConn, err := net.Dial("tcp", "127.0.0.1:5900")
	if err != nil {
		fmt.Printf("Dial fake failed : %v\n", err)
		return
	}

	if err != nil {
		return
	}
	go io.Copy(fakeConn, conn)
	go io.Copy(conn, fakeConn)
}

func TestSmuxServer(t *testing.T) {
	listen, err := net.Listen("tcp", "0.0.0.0:7556")
	if err != nil {
		fmt.Printf("Start server failed : %v\n", err)
		return
	}
	defer listen.Close()
	for {
		conn, err := listen.Accept()
		if err != nil {
			fmt.Printf("Accept error: %v\n", err)
			continue
		}
		session, err := smux.Server(conn, nil)
		if err != nil {
			fmt.Printf("smux error: %v\n", err)
			continue
		}

		go func() {
			for {
				stream, err := session.AcceptStream()
				if err != nil {
					fmt.Printf("AcceptStream error: %v\n", err)
					continue
				}
				fmt.Println("A")
				go handlerMuxTest(stream)
			}
		}()
	}

}

func TestSmuxClient(t *testing.T) {
	listen, err := net.Listen("tcp", "0.0.0.0:11222")
	if err != nil {
		fmt.Printf("Start server failed : %v\n", err)
		return
	}
	defer listen.Close()

	smuxConn, err := net.Dial("tcp", "127.0.0.1:7556")
	if err != nil {
		fmt.Printf("Start smuxConn failed : %v\n", err)
		return
	}
	session, err := smux.Client(smuxConn, nil)
	if err != nil {
		fmt.Printf("Start smux.Client failed : %v\n", err)
		return
	}
	for {
		conn, err := listen.Accept()
		if err != nil {
			fmt.Printf("Accept error: %v\n", err)
			continue
		}
		fmt.Println("A")

		stream, err := session.OpenStream()
		if err != nil {
			fmt.Printf("OpenStream error: %v\n", err)
			continue

		}
		go io.Copy(conn, stream)
		go io.Copy(stream, conn)
	}
}

func handlerMuxTest(conn *smux.Stream) {

	fakeConn, err := net.Dial("tcp", "127.0.0.1:5900")
	if err != nil {
		fmt.Printf("Dial fake failed : %v\n", err)
		return
	}
	fmt.Println("UUUUUUUUUUUUUU")
	if err != nil {
		return
	}
	go io.Copy(fakeConn, conn)
	go io.Copy(conn, fakeConn)
}
